@@ -1,10 +1,14 @@
 package File::Slurper;
-$File::Slurper::VERSION = '0.007';
+$File::Slurper::VERSION = '0.012';
 use strict;
 use warnings;
 
 use Carp 'croak';
 use Exporter 5.57 'import';
+
+use Encode 2.11 qw/FB_CROAK STOP_AT_PARTIAL/;
+use PerlIO::encoding;
+
 our @EXPORT_OK = qw/read_binary read_text read_lines write_binary write_text read_dir/;
 
 sub read_binary {
@@ -29,17 +33,14 @@ sub read_binary {
 
 use constant {
 	CRLF_DEFAULT => $^O eq 'MSWin32',
-	HAS_UTF8_STRICT => do { local $@; eval { require PerlIO::utf8_strict } },
+	HAS_UTF8_STRICT => scalar do { local $@; eval { require PerlIO::utf8_strict } },
 };
 
 sub _text_layers {
 	my ($encoding, $crlf) = @_;
 	$crlf = CRLF_DEFAULT if $crlf && $crlf eq 'auto';
 
-	if ($encoding =~ /^(latin|iso-8859-)1$/i) {
-		return $crlf ? ':unix:crlf' : ':raw';
-	}
-	elsif (HAS_UTF8_STRICT && $encoding =~ /^utf-?8\b/i) {
+	if (HAS_UTF8_STRICT && $encoding =~ /^utf-?8\b/i) {
 		return $crlf ? ':unix:utf8_strict:crlf' : ':unix:utf8_strict';
 	}
 	else {
@@ -52,9 +53,8 @@ sub read_text {
 	my ($filename, $encoding, $crlf) = @_;
 	$encoding ||= 'utf-8';
 	my $layer = _text_layers($encoding, $crlf);
-	return read_binary($filename) if $layer eq ':raw';
 
-	local $PerlIO::encoding::fallback = 1;
+	local $PerlIO::encoding::fallback = STOP_AT_PARTIAL | FB_CROAK;
 	open my $fh, "<$layer", $filename or croak "Couldn't open $filename: $!";
 	return do { local $/; <$fh> };
 }
@@ -64,7 +64,7 @@ sub write_text {
 	$encoding ||= 'utf-8';
 	my $layer = _text_layers($encoding, $crlf);
 
-	local $PerlIO::encoding::fallback = 1;
+	local $PerlIO::encoding::fallback = STOP_AT_PARTIAL | FB_CROAK;
 	open my $fh, ">$layer", $filename or croak "Couldn't open $filename: $!";
 	print $fh $_[1] or croak "Couldn't write to $filename: $!";
 	close $fh or croak "Couldn't write to $filename: $!";
@@ -72,7 +72,11 @@ sub write_text {
 }
 
 sub write_binary {
-	return write_text(@_[0,1], 'latin-1');
+	my $filename = $_[0];
+	open my $fh, ">:raw", $filename or croak "Couldn't open $filename: $!";
+	print $fh $_[1] or croak "Couldn't write to $filename: $!";
+	close $fh or croak "Couldn't write to $filename: $!";
+	return;
 }
 
 sub read_lines {
@@ -80,7 +84,7 @@ sub read_lines {
 	$encoding ||= 'utf-8';
 	my $layer = _text_layers($encoding, $crlf);
 
-	local $PerlIO::encoding::fallback = 1;
+	local $PerlIO::encoding::fallback = STOP_AT_PARTIAL | FB_CROAK;
 	open my $fh, "<$layer", $filename or croak "Couldn't open $filename: $!";
 	return <$fh> if $skip_chomp;
 	my @buf = <$fh>;
@@ -111,7 +115,7 @@ File::Slurper - A simple, sane and efficient module to slurp a file
 
 =head1 VERSION
 
-version 0.007
+version 0.012
 
 =head1 SYNOPSIS
 
@@ -120,7 +124,7 @@ version 0.007
 
 =head1 DESCRIPTION
 
-This module provides functions for fast and correct slurping and spewing. All functions are optionally exported.
+This module provides functions for fast and correct slurping and spewing. All functions are optionally exported. All functions throw exceptions on errors, write functions don't return any meaningful value.
 
 =head1 FUNCTIONS
 
@@ -170,7 +174,7 @@ An attempt to expose as many IO related features as possible via a single API.
 
 =item * L<File::Slurp|File::Slurp>
 
-This is previous generation file slurping module. It has a number of issues, as described <here|http://blogs.perl.org/users/leon_timmermans/2015/08/fileslurp-is-broken-and-wrong.html>
+This is a previous generation file slurping module. It has a number of issues, as described L<here|http://blogs.perl.org/users/leon_timmermans/2015/08/fileslurp-is-broken-and-wrong.html>.
 
 =item * L<File::Slurp::Tiny|File::Slurp::Tiny>
 
